package emfer.stories;

import com.google.common.base.Objects;
import emfer.reachability.ReachabilityGraph;
import emfer.reachability.ReachableState;
import emfer.reachability.TrafoApplication;
import emfer.stories.Edge;
import java.util.ArrayList;
import java.util.LinkedHashSet;
import java.util.Set;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.xtend2.lib.StringConcatenation;

@SuppressWarnings("all")
public class StoryStep {
  public String content;
  
  private ReachabilityGraph reachabilityGraph;
  
  private ReachableState reachableState;
  
  @Override
  public String toString() {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("<p>");
    _builder.newLine();
    _builder.append("<pre>");
    _builder.newLine();
    _builder.append(this.content);
    _builder.newLineIfNotEmpty();
    _builder.append("</pre>");
    _builder.newLine();
    _builder.append("</p>");
    _builder.newLine();
    {
      boolean _notEquals = (!Objects.equal(this.reachabilityGraph, null));
      if (_notEquals) {
        String _addReachabilityGraph = this.addReachabilityGraph();
        _builder.append(_addReachabilityGraph);
        _builder.newLineIfNotEmpty();
      }
    }
    {
      boolean _notEquals_1 = (!Objects.equal(this.reachableState, null));
      if (_notEquals_1) {
        CharSequence _addReachableState = this.addReachableState();
        _builder.append(_addReachableState);
        _builder.newLineIfNotEmpty();
      }
    }
    return _builder.toString();
  }
  
  public String genHtmlStateDescription(final ReachableState state, final String nameSpace) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("<p id=\"");
    _builder.append(nameSpace);
    long _number = state.getNumber();
    _builder.append(_number);
    _builder.append("\"> </p>");
    String result = _builder.toString();
    EList<TrafoApplication> _resultOf = state.getResultOf();
    for (final TrafoApplication t : _resultOf) {
      {
        ReachableState src = t.getSrc();
        StringConcatenation _builder_1 = new StringConcatenation();
        _builder_1.append(result);
        _builder_1.newLineIfNotEmpty();
        _builder_1.append("<p><a href=\"#");
        _builder_1.append(nameSpace);
        long _number_1 = src.getNumber();
        _builder_1.append(_number_1);
        _builder_1.append("\">");
        long _number_2 = src.getNumber();
        _builder_1.append(_number_2);
        _builder_1.append("</a> --");
        String _description = t.getDescription();
        _builder_1.append(_description);
        _builder_1.append("-+ ");
        long _number_3 = state.getNumber();
        _builder_1.append(_number_3);
        _builder_1.newLineIfNotEmpty();
        result = _builder_1.toString();
      }
    }
    StringConcatenation _builder_1 = new StringConcatenation();
    _builder_1.append(result);
    _builder_1.newLineIfNotEmpty();
    _builder_1.append("<p>");
    _builder_1.newLine();
    _builder_1.append(state);
    _builder_1.newLineIfNotEmpty();
    _builder_1.append("</p>");
    _builder_1.newLine();
    result = _builder_1.toString();
    EList<TrafoApplication> _trafoApplications = state.getTrafoApplications();
    for (final TrafoApplication t_1 : _trafoApplications) {
      {
        ReachableState tgt = t_1.getTgt();
        StringConcatenation _builder_2 = new StringConcatenation();
        _builder_2.append(result);
        _builder_2.newLineIfNotEmpty();
        _builder_2.append("<p>");
        long _number_1 = state.getNumber();
        _builder_2.append(_number_1);
        _builder_2.append(" --");
        String _description = t_1.getDescription();
        _builder_2.append(_description);
        _builder_2.append("-+ <a href=\"#");
        _builder_2.append(nameSpace);
        long _number_2 = tgt.getNumber();
        _builder_2.append(_number_2);
        _builder_2.append("\">");
        long _number_3 = tgt.getNumber();
        _builder_2.append(_number_3);
        _builder_2.append("</a> metricValue: ");
        double _metricValue = tgt.getMetricValue();
        _builder_2.append(_metricValue);
        _builder_2.newLineIfNotEmpty();
        result = _builder_2.toString();
      }
    }
    return result;
  }
  
  public CharSequence addReachableState() {
    CharSequence _xblockexpression = null;
    {
      LinkedHashSet<EObject> nodes = new LinkedHashSet<EObject>();
      ArrayList<Edge> edges = new ArrayList<Edge>();
      nodes.add(this.reachableState.getRoot());
      this.aggregate(nodes, edges);
      StringConcatenation _builder = new StringConcatenation();
      _builder.newLine();
      _builder.append("<div class=\"alchemy\" id=\"alchemy_");
      String _string = Integer.valueOf(this.hashCode()).toString();
      _builder.append(_string);
      _builder.append("\"></div>");
      _builder.newLineIfNotEmpty();
      _builder.append("<script type=\"text/javascript\">");
      _builder.newLine();
      _builder.newLine();
      _builder.append("var data= {");
      _builder.newLine();
      {
        boolean _hasElements = false;
        for(final EObject n : nodes) {
          if (!_hasElements) {
            _hasElements = true;
            _builder.append("\"nodes\": [\n");
          } else {
            _builder.appendImmediate(",", "");
          }
          _builder.append("{type:\"state\",id:");
          int _hashCode = n.hashCode();
          _builder.append(_hashCode);
          _builder.append(",caption:\"");
          String _replaceAll = n.toString().replaceAll("\n", ", ");
          _builder.append(_replaceAll);
          _builder.append("\"}");
          _builder.newLineIfNotEmpty();
        }
        if (_hasElements) {
          _builder.append("]");
        }
      }
      _builder.append(",");
      int i = 0;
      _builder.newLineIfNotEmpty();
      {
        boolean _hasElements_1 = false;
        for(final Edge e : edges) {
          if (!_hasElements_1) {
            _hasElements_1 = true;
            _builder.append("\"edges\": [\n");
          } else {
            _builder.appendImmediate(",", "");
          }
          _builder.append("{source:");
          int _hashCode_1 = e.source.hashCode();
          _builder.append(_hashCode_1);
          _builder.append(",target:");
          int _hashCode_2 = e.target.hashCode();
          _builder.append(_hashCode_2);
          _builder.append(",nr:");
          int _plusPlus = i++;
          _builder.append(_plusPlus);
          _builder.append(",caption:\"");
          String _name = e.edge.getName();
          _builder.append(_name);
          _builder.append("\",type:\"normal\"}");
          _builder.newLineIfNotEmpty();
        }
        if (_hasElements_1) {
          _builder.append("]");
        }
      }
      _builder.append("}");
      _builder.newLine();
      CharSequence _config = this.getConfig();
      _builder.append(_config);
      _builder.newLineIfNotEmpty();
      _builder.append("alchemy = new Alchemy(config)");
      _builder.newLine();
      _builder.append("</script>");
      _builder.newLine();
      _xblockexpression = _builder;
    }
    return _xblockexpression;
  }
  
  public String addReachabilityGraph() {
    StringConcatenation _builder = new StringConcatenation();
    _builder.newLine();
    _builder.append("<div class=\"alchemy\" id=\"alchemy_");
    String _string = Integer.valueOf(this.hashCode()).toString();
    _builder.append(_string);
    _builder.append("\"></div>");
    _builder.newLineIfNotEmpty();
    _builder.append("<script type=\"text/javascript\">");
    _builder.newLine();
    _builder.newLine();
    _builder.append("var data= {");
    _builder.newLine();
    {
      EList<ReachableState> _states = this.reachabilityGraph.getStates();
      boolean _hasElements = false;
      for(final ReachableState s : _states) {
        if (!_hasElements) {
          _hasElements = true;
          _builder.append("\"nodes\": [\n");
        } else {
          _builder.appendImmediate(",", "");
        }
        _builder.append("{type:\"");
        String _type = this.type(s);
        _builder.append(_type);
        _builder.append("\",id:");
        long _number = s.getNumber();
        _builder.append(_number);
        _builder.append(",caption:\"state ");
        long _number_1 = s.getNumber();
        _builder.append(_number_1);
        _builder.append("\"}");
        _builder.newLineIfNotEmpty();
      }
      if (_hasElements) {
        _builder.append("]");
      }
    }
    _builder.append(",");
    int i = 0;
    _builder.newLineIfNotEmpty();
    {
      EList<TrafoApplication> _trafoApplications = this.reachabilityGraph.getTrafoApplications();
      boolean _hasElements_1 = false;
      for(final TrafoApplication t : _trafoApplications) {
        if (!_hasElements_1) {
          _hasElements_1 = true;
          _builder.append("\"edges\": [\n");
        } else {
          _builder.appendImmediate(",", "");
        }
        _builder.append("{source:");
        long _number_2 = t.getSrc().getNumber();
        _builder.append(_number_2);
        _builder.append(",target:");
        long _number_3 = t.getTgt().getNumber();
        _builder.append(_number_3);
        _builder.append(",nr:");
        int _plusPlus = i++;
        _builder.append(_plusPlus);
        _builder.append(",caption:\"");
        String _description = t.getDescription();
        _builder.append(_description);
        _builder.append("\",type:\"normal\"}");
        _builder.newLineIfNotEmpty();
      }
      if (_hasElements_1) {
        _builder.append("]");
      }
    }
    _builder.append("}");
    _builder.newLine();
    CharSequence _config = this.getConfig();
    _builder.append(_config);
    _builder.newLineIfNotEmpty();
    _builder.append("alchemy = new Alchemy(config)");
    _builder.newLine();
    _builder.append("</script>");
    _builder.newLine();
    return _builder.toString();
  }
  
  public CharSequence getConfig() {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("var config = {");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("dataSource: data,");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("divSelector: alchemy_");
    String _string = Integer.valueOf(this.hashCode()).toString();
    _builder.append(_string, "\t\t");
    _builder.append(",");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t  ");
    _builder.append("directedEdges:true,");
    _builder.newLine();
    _builder.append("\t\t  ");
    _builder.append("graphWidth: function() {return screen.availWidth  * 0.7},");
    _builder.newLine();
    _builder.append("\t\t  ");
    _builder.append("graphHeight: function() {return screen.availHeight * 0.7},");
    _builder.newLine();
    _builder.append("\t\t  ");
    _builder.append("initialScale: 0.5, ");
    _builder.newLine();
    _builder.append("\t\t  ");
    _builder.append("initialTranslate: [250,250],");
    _builder.newLine();
    _builder.append("\t\t  ");
    _builder.append("curvedEdges:false,");
    _builder.newLine();
    _builder.append("\t\t  ");
    _builder.append("edgeCaptionsOnByDefault: true,");
    _builder.newLine();
    _builder.append("\t\t  ");
    _builder.append("backgroundColour: \"#FFFFFF\",");
    _builder.newLine();
    _builder.append("\t\t  ");
    _builder.append("nodeCaptionsOnByDefault: true,");
    _builder.newLine();
    _builder.append("\t\t  ");
    _builder.append("nodeTypes: {\"type\":[\"failure\",\"state\",\"startstate\",\"finalstate\"]},");
    _builder.newLine();
    _builder.append("\t\t   ");
    _builder.append("\"nodeStyle\": {");
    _builder.newLine();
    _builder.append("\t\t   ");
    _builder.append("\"failure\": {");
    _builder.newLine();
    _builder.append("\t\t         ");
    _builder.append("color: \"#FE3E3E\",");
    _builder.newLine();
    _builder.append("\t\t      ");
    _builder.append("},");
    _builder.newLine();
    _builder.append("\t\t   ");
    _builder.append("\"finalstate\": {");
    _builder.newLine();
    _builder.append("\t\t         ");
    _builder.append("color: \"#0000FF\",");
    _builder.newLine();
    _builder.append("\t\t         ");
    _builder.append("radius: 15, ");
    _builder.newLine();
    _builder.append("\t\t      ");
    _builder.append("borderColor: \"#127DC1\"            },");
    _builder.newLine();
    _builder.append("\t\t   ");
    _builder.append("\"startstate\": {");
    _builder.newLine();
    _builder.append("\t\t         ");
    _builder.append("color: \"#FFFFFF\",");
    _builder.newLine();
    _builder.append("\t\t         ");
    _builder.append("radius: 15, ");
    _builder.newLine();
    _builder.append("\t\t      ");
    _builder.append("borderColor: \"#127DC1\"            },");
    _builder.newLine();
    _builder.append("\t\t   ");
    _builder.append("\"state\":{");
    _builder.newLine();
    _builder.append("\t\t         ");
    _builder.append("color: \"#bd59f7\",");
    _builder.newLine();
    _builder.append("\t\t         ");
    _builder.append("borderColor: \"#9018e0\",");
    _builder.newLine();
    _builder.append("\t\t         ");
    _builder.append("captionColor: \"#FFFFFF\"");
    _builder.newLine();
    _builder.append("\t\t   ");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t\t  ");
    _builder.append("},");
    _builder.newLine();
    _builder.append("\t\t  ");
    _builder.append("\"edgeTypes\": {\"type\": [\"failure\",\"normal\"]},");
    _builder.newLine();
    _builder.append("\t\t  ");
    _builder.append("\"edgeStyle\": {");
    _builder.newLine();
    _builder.append("\t\t      ");
    _builder.append("\"failure\": {");
    _builder.newLine();
    _builder.append("\t\t         ");
    _builder.append("color: \"#ff794d\",");
    _builder.newLine();
    _builder.append("\t\t      ");
    _builder.append("width: 3,");
    _builder.newLine();
    _builder.append("\t\t         ");
    _builder.append("opacity: 1");
    _builder.newLine();
    _builder.append("\t\t   ");
    _builder.append("},");
    _builder.newLine();
    _builder.append("\t\t      ");
    _builder.append("\"greenpath\": {");
    _builder.newLine();
    _builder.append("\t\t         ");
    _builder.append("color: \"#00FF00\",");
    _builder.newLine();
    _builder.append("\t\t      ");
    _builder.append("width: 7,");
    _builder.newLine();
    _builder.append("\t\t         ");
    _builder.append("opacity: 1");
    _builder.newLine();
    _builder.append("\t\t   ");
    _builder.append("},");
    _builder.newLine();
    _builder.append("\t\t   ");
    _builder.append("\"normal\": {");
    _builder.newLine();
    _builder.append("\t\t      ");
    _builder.append("opacity: .9,");
    _builder.newLine();
    _builder.append("\t\t      ");
    _builder.append("color: \"#575459\",");
    _builder.newLine();
    _builder.append("\t\t      ");
    _builder.append("width: 3");
    _builder.newLine();
    _builder.append("\t\t   ");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t\t  ");
    _builder.append("},     ");
    _builder.newLine();
    _builder.append("\t\t   ");
    _builder.append("nodeClick: function OpenInNewTab(node) {");
    _builder.newLine();
    _builder.append("\t\t         ");
    _builder.append("var win = window.open(\'rwp-images/state\'+node.id+\'.png\', \'_blank\');");
    _builder.newLine();
    _builder.append("\t\t         ");
    _builder.append("if (win) {");
    _builder.newLine();
    _builder.append("\t\t               ");
    _builder.append("win.focus();");
    _builder.newLine();
    _builder.append("\t\t         ");
    _builder.append("} else {");
    _builder.newLine();
    _builder.append("\t\t               ");
    _builder.append("alert(\'Please allow popups for this website\');");
    _builder.newLine();
    _builder.append("\t\t         ");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t\t      ");
    _builder.append("}     ");
    _builder.newLine();
    _builder.append("\t\t\t");
    _builder.append(",     ");
    _builder.newLine();
    _builder.append("\t\t\t");
    _builder.append("edgeClick: function OpenInNewTab(edge) {");
    _builder.newLine();
    _builder.append("\t\t\t    ");
    _builder.append("var win = window.open(\'rulegraphs/graph\'+edge._properties.nr+\'.html\', \'_blank\');");
    _builder.newLine();
    _builder.append("\t\t\t    ");
    _builder.append("if (win) {");
    _builder.newLine();
    _builder.append("\t\t\t          ");
    _builder.append("win.focus();");
    _builder.newLine();
    _builder.append("\t\t\t    ");
    _builder.append("} else {");
    _builder.newLine();
    _builder.append("\t\t\t          ");
    _builder.append("alert(\'Please allow popups for this website\');");
    _builder.newLine();
    _builder.append("\t\t\t    ");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t\t\t ");
    _builder.append("}     ");
    _builder.newLine();
    _builder.append("\t\t\t    ");
    _builder.append("};");
    _builder.newLine();
    return _builder;
  }
  
  public String type(final ReachableState state) {
    String _xblockexpression = null;
    {
      double _metricValue = state.getMetricValue();
      int i = ((int) _metricValue);
      String _switchResult = null;
      switch (i) {
        case 0:
          StringConcatenation _builder = new StringConcatenation();
          _builder.append("finalstate");
          _switchResult = _builder.toString();
          break;
        default:
          StringConcatenation _builder_1 = new StringConcatenation();
          _builder_1.append("state");
          _switchResult = _builder_1.toString();
          break;
      }
      _xblockexpression = _switchResult;
    }
    return _xblockexpression;
  }
  
  public ReachabilityGraph setGraph(final ReachabilityGraph graph) {
    return this.reachabilityGraph = graph;
  }
  
  public ReachableState setReachableState(final ReachableState state) {
    return this.reachableState = state;
  }
  
  public void aggregate(final Set<EObject> dynNodes, final ArrayList<Edge> dynEdges) {
    ArrayList<EObject> todo = new ArrayList<EObject>();
    todo.addAll(dynNodes);
    while ((!todo.isEmpty())) {
      {
        int _size = todo.size();
        int _minus = (_size - 1);
        EObject eObject = todo.remove(_minus);
        EClass eClass = eObject.eClass();
        EList<EStructuralFeature> _eAllStructuralFeatures = eClass.getEAllStructuralFeatures();
        for (final EStructuralFeature f : _eAllStructuralFeatures) {
          {
            Object value = eObject.eGet(f);
            if ((value instanceof EList)) {
              for (final Object elem : ((EList)value)) {
                if (((!Objects.equal(elem, null)) && (elem instanceof EObject))) {
                  Edge _edge = new Edge(eObject, ((EObject) elem), f);
                  dynEdges.add(_edge);
                  boolean _contains = dynNodes.contains(elem);
                  boolean _not = (!_contains);
                  if (_not) {
                    EObject e = ((EObject) elem);
                    dynNodes.add(e);
                    todo.add(e);
                  }
                }
              }
            } else {
              if (((!Objects.equal(value, null)) && (value instanceof EObject))) {
                Edge _edge_1 = new Edge(eObject, ((EObject) value), f);
                dynEdges.add(_edge_1);
                boolean _contains_1 = dynNodes.contains(value);
                boolean _not_1 = (!_contains_1);
                if (_not_1) {
                  EObject e_1 = ((EObject) value);
                  dynNodes.add(e_1);
                  todo.add(e_1);
                }
              }
            }
          }
        }
        EObject eContainer = eObject.eContainer();
        boolean _notEquals = (!Objects.equal(eContainer, null));
        if (_notEquals) {
          EStructuralFeature _eContainingFeature = eObject.eContainingFeature();
          Edge _edge = new Edge(eObject, eContainer, _eContainingFeature);
          dynEdges.add(_edge);
        }
      }
    }
  }
}
