package de.tudresden.inf.st.mquat.solving.genetic.opt4j.custom.optimizer;

import com.google.inject.Inject;
import com.google.inject.Singleton;
import de.tudresden.inf.st.mquat.solving.genetic.GeneticSolver;
import de.tudresden.inf.st.mquat.utils.StopWatch;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.opt4j.core.optimizer.*;

import java.util.concurrent.TimeUnit;

/**
 * The {@link org.opt4j.core.optimizer.OptimizationMediator} performs the overall optimization process
 * for the {@link IterativeOptimizer}.
 *
 * @author reimann, glass, lukasiewycz
 *
 */
@Singleton
public class TimeOutOptimizer extends AbstractOptimizer {

  private static final Logger logger = LogManager.getLogger(TimeOutOptimizer.class);

  protected final IterativeOptimizer iterativeOptimizer;

  /**
   * Creates a new {@link org.opt4j.core.optimizer.OptimizationMediator}.
   *
   * @param iterativeOptimizer
   *            the iterative optimizer to use
   * @param population
   *            the specified population
   * @param archive
   *            the specified archive
   * @param completer
   *            the specified completer
   * @param control
   *            the control
   * @param iteration
   *            the iteration counter
   */
  @Inject
  public TimeOutOptimizer(IterativeOptimizer iterativeOptimizer, Population population, Archive archive,
                              IndividualCompleter completer, Control control, TimeOutIteration iteration) {
    super(population, archive, completer, control, iteration);
    this.iterativeOptimizer = iterativeOptimizer;
  }

  /*
   * (non-Javadoc)
   *
   * @see org.opt4j.core.optimizer.Optimizer#optimize()
   */
  @Override
  public void optimize() throws StopException, TerminationException {
    iterativeOptimizer.initialize();
    StopWatch watch = StopWatch.start();
    while (iteration.value() < iteration.max() && watch.time(TimeUnit.MILLISECONDS) < ((TimeOutIteration)iteration).maxTime()) {
      iterativeOptimizer.next();
      nextIteration();
    }
    logger.info("Stopped after {} iterations.", iteration.value());
  }
}
